#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Este script envía mediante UFTP la imagen recibida como primer parámetro, al puerto e IP (Multicast o Unicast) especificados en el segundo parámetro,
  a la velocidad de transferencia tambíén especificada en el segundo parámetro (la sintaxis de este parámetro es "Port:IP:Bitrate").
Previamente, los clientes deben haberse puesto a escuchar en la IP Multicast correspondiente (tanto para envíos Multicast como para envíos Unicast).

Paquetes APT requeridos: "uftp"

 Parámetros
------------
sys.argv[1] - Nombre completo de la imagen a enviar (con o sin ruta), pero incluyendo el subdirectorio correspondiente a la OU, si es el caso.
            - Ejemplo1: image1.img
            - Ejemplo2: /opt/opengnsys/images/image1.img
            - Ejemplo3: ou_subdir/image1.img
            - Ejemplo4: /ou_subdir/image1.img
            - Ejemplo5: /opt/opengnsys/images/ou_subdir/image1.img

sys.argv[2] - Parámetros Multicast/Unicast (en formato "Port:IP:Bitrate")
            - Ejemplo1: 9000:239.194.17.2:100M
            - Ejemplo2: 9000:192.168.56.101:1G

 Sintaxis
----------
./sendFileUFTP.py [ou_subdir/]image_name|/image_path/image_name Port:IP:Bitrate

 Ejemplos
 ---------
./sendFileUFTP.py image1.img 9000:239.194.17.2:100M
./sendFileUFTP.py /opt/opengnsys/images/image1.img 9000:239.194.17.2:100M
./sendFileUFTP.py ou_subdir/image1.img 9000:192.168.56.101:1G
./sendFileUFTP.py /ou_subdir/image1.img 9000:192.168.56.101:1G
./sendFileUFTP.py /opt/opengnsys/images/ou_subdir/image1.img 9000:192.168.56.101:1G
"""

# --------------------------------------------------------------------------------------------
#                                        IMPORTS
# --------------------------------------------------------------------------------------------

import os
import sys
import subprocess


# --------------------------------------------------------------------------------------------
#                                        VARIABLES
# --------------------------------------------------------------------------------------------

script_name = os.path.basename(__file__)
repo_path = '/opt/opengnsys/images/'
log_file = '/opt/opengnsys/log/uftp.log'


# --------------------------------------------------------------------------------------------
#                                        FUNCTIONS
# --------------------------------------------------------------------------------------------


def show_help():
    """ Imprime la ayuda, cuando se ejecuta el script con el parámetro "help".
    """
    help_text = f"""
        Sintaxis: {script_name} [ou_subdir/]image_name|/image_path/image_name Port:IP:Bitrate
        Ejemplo1: {script_name} image1.img 9000:239.194.17.2:100M
        Ejemplo2: {script_name} /opt/opengnsys/images/image1.img 9000:239.194.17.2:100M
        Ejemplo3: {script_name} ou_subdir/image1.img 9000:192.168.56.101:1G
        Ejemplo4: {script_name} /ou_subdir/image1.img 9000:192.168.56.101:1G
        Ejemplo5: {script_name} /opt/opengnsys/images/ou_subdir/image1.img 9000:192.168.56.101:1G
        """
    print(help_text)


def check_params():
    """ Comprueba que se haya enviado la cantidad correcta de parámetros, y en el formato correcto.
        Si no es así, muestra un mensaje de error, y sale del script.
        LLama a la función "show_help" cuando se ejecuta el script con el parámetro "help".
    """
    # Si se ejecuta el script con el parámetro "help", se muestra la ayuda, y se sale del script:
    if len(sys.argv) == 2 and sys.argv[1] == "help":
        show_help()
        sys.exit(0)
    # Si se ejecuta la función con más o menos de 2 parámetros, se muestra un mensaje de error, y se sale del script:
    elif len(sys.argv) != 3:
        print(f"{script_name} Error: Formato incorrecto: Se debe especificar 2 parámetros (image_name|/image_path/image_name port:ip:bitrate)")
        sys.exit(1)
    # Si en el segundo parámetro no hay 3 elementos (separados por ":"), se muestra un mensaje de error, y se sale del script:
    param_list = sys.argv[2].split(':')
    if len(param_list) != 3:
        print(f"{script_name} Error: Datos Multicast incorrectos: \"{sys.argv[2]}\" (se debe especificar \"puerto:ip:bitrate\")")
        sys.exit(3)


def build_file_path():
    """ Construye la ruta completa al archivo a enviar
          (agregando "/opt/opengnsys/images/" si no se ha especificado en el parámetro).
    """
    param_path = sys.argv[1]
    # Si la ruta comienza con una barra, pero que no corresponde a "repo_path"
    #   (porque corresponderá al subdirectorio de una OU), eliminamos la barra:
    if param_path.startswith('/') and not param_path.startswith(repo_path):
        param_path = param_path.lstrip('/')
    # Construimos la ruta completa:
    if not param_path.startswith(repo_path):
        file_path = os.path.join(repo_path, param_path)
    else:
        file_path = param_path
    return file_path


def calculate_bitrate(bitrate):
    """ Calcula el valor de la variable "bitrate", en base a la letra especificada ("K", "M" o "G").
        Luego elimina la letra y retorna el valor con tipo de datos "int".
    """
    bitrate = bitrate.lower()
    if "m" in bitrate:
        bitrate = int(bitrate.strip("m")) * 1024
    elif "g" in bitrate:
        bitrate = int(bitrate.strip("g")) * 1024 * 1024
    else:
        bitrate = int(bitrate.strip("k"))
    return bitrate



# --------------------------------------------------------------------------------------------
#                                          MAIN
# --------------------------------------------------------------------------------------------


def main():
    """
    """
    # Evaluamos si se ha enviado la cantidad correcta de parámetros, y en el formato correcto:
    check_params()
    
    # Obtenemos la ruta completa al archivo a enviar:
    file_path = build_file_path()

    # Si el fichero no es accesible, devolvermos un error, y salimos del script:
    if not os.path.isfile(file_path):
        print(f"{script_name} Error: Fichero \"{file_path}\" no accesible")
        sys.exit(2)

    # Almacenamos los elementos del segundo parámetro en variables (su formato es "puerto:ip:bitrate"):
    param_list = sys.argv[2].split(':')
    port, ip, bitrate = param_list

    # Calculamos el valor de la variable "bitrate", en base a la letra especificada (que luego eliminamos de la variable):
    bitrate = calculate_bitrate(bitrate)

    # Creamos una lista con el comando a enviar (esto es requerido por la función "subprocess.run"), e imprimimos el comando con espacios:
    splitted_cmd = f"uftp -M {ip} -p {port} -L {log_file} -Y aes256-cbc -h sha256 -e rsa -c -K 1024 -R {bitrate} {file_path}".split()

    print(f"Sending command: {' '.join(splitted_cmd)}")

    # Ejecutamos el comando en el sistema, e imprimimos el resultado:
    try:
        result = subprocess.run(splitted_cmd, check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        print(f"ReturnCode: {result.returncode}")
    except subprocess.CalledProcessError as error:
        print(f"ReturnCode: {error.returncode}")
        print(f"Error Output: {error.stderr.decode()}")
    except Exception as error:
        print(f"Se ha producido un error inesperado: {error}")



# --------------------------------------------------------------------------------------------

if __name__ == "__main__":
    main()

# --------------------------------------------------------------------------------------------
