#!/usr/bin/env python3
# -*- coding: utf-8 -*-


"""
Este script finaliza el proceso "udp-sender" asociado a la imagen que recibe como parámetro,
  lo que en la práctica hará que se cancele la transmisión existente de dicha imagen mediante UDPcast.

 Parámetros
------------
sys.argv[1] - Nombre completo de la imagen a cancelar su transmisión (con o sin ruta), pero incluyendo el subdirectorio correspondiente a la OU, si es el caso.
            - Ejemplo1: image1.img
            - Ejemplo2: /opt/opengnsys/images/image1.img
            - Ejemplo3: ou_subdir/image1.img
            - Ejemplo4: /ou_subdir/image1.img
            - Ejemplo5: /opt/opengnsys/images/ou_subdir/image1.img

 Sintaxis
----------
./stopUDPcast.py [ou_subdir/]image_name|/image_path/image_name

 Ejemplos
 ---------
./stopUDPcast.py image1.img
./stopUDPcast.py /opt/opengnsys/images/image1.img
./stopUDPcast.py ou_subdir/image1.img
./stopUDPcast.py /ou_subdir/image1.img
./stopUDPcast.py /opt/opengnsys/images/ou_subdir/image1.img
"""

# --------------------------------------------------------------------------------------------
#                                        IMPORTS
# --------------------------------------------------------------------------------------------

import os
import sys
import subprocess


# --------------------------------------------------------------------------------------------
#                                        VARIABLES
# --------------------------------------------------------------------------------------------

script_name = os.path.basename(__file__)
repo_path = '/opt/opengnsys/images/'


# --------------------------------------------------------------------------------------------
#                                        FUNCTIONS
# --------------------------------------------------------------------------------------------


def show_help():
    """ Imprime la ayuda, cuando se ejecuta el script con el parámetro "help".
    """
    help_text = f"""
        Sintaxis: {script_name} [ou_subdir/]image_name|/image_path/image_name
        Ejemplo1: {script_name} image1.img
        Ejemplo2: {script_name} /opt/opengnsys/images/image1.img
        Ejemplo3: {script_name} ou_subdir/image1.img
        Ejemplo4: {script_name} /ou_subdir/image1.img
        Ejemplo5: {script_name} /opt/opengnsys/images/ou_subdir/image1.img
        """
    print(help_text)


def check_params():
    """ Comprueba que se haya enviado la cantidad correcta de parámetros, y en el formato correcto.
        Si no es así, muestra un mensaje de error, y sale del script.
        LLama a la función "show_help" cuando se ejecuta el script con el parámetro "help".
    """
    # Si se ejecuta el script con el parámetro "help", se muestra la ayuda, y se sale del script:
    if len(sys.argv) == 2 and sys.argv[1] == "help":
        show_help()
        sys.exit(0)
    # Si se ejecuta el script con más o menos de 1 parámetro, se muestra un error y la ayuda, y se sale del script:
    elif len(sys.argv) != 2:
        print(f"{script_name} Error: Formato incorrecto: Se debe especificar 1 parámetro")
        show_help()
        sys.exit(1)


def build_file_path():
    """ Construye la ruta completa del archivo a chequear
          (agregando "/opt/opengnsys/images/" si no se ha especificado en el parámetro).
    """
    param_path = sys.argv[1]
    # Si la ruta comienza con una barra, pero que no corresponde a "repo_path"
    #   (porque corresponderá al subdirectorio de una OU), eliminamos la barra:
    if param_path.startswith('/') and not param_path.startswith(repo_path):
        param_path = param_path.lstrip('/')
    # Construimos la ruta completa:
    if not param_path.startswith(repo_path):
        file_path = os.path.join(repo_path, param_path)
    else:
        file_path = param_path
    return file_path


def get_process_pid(file_path):
    """ Busca un proceso que contenga "udp-sender" y la ruta de la imagen que recibe como parámetro,
          y si lo encuentra almacena y retorna su pid asociado.
        Si no encuentra ningún proceso que cumpla las condiciones (o si se produce una excepción) sale del script.
    """
    try:
        # Obtenemos todos los procesos, y almacenamos la salida y los errores:
        result = subprocess.Popen(['ps', '-aux'], stdout=subprocess.PIPE, stderr=subprocess.PIPE, encoding='UTF8')
        out, error = result.communicate()

        # Almacenamos en una lista los procesos que contengan "udp-sender" y la imagen especificada como parámetro:
        filtered_lines = [line for line in out.split('\n') if 'udp-sender' in line and file_path in line]
        # Si hemos encontrado un proceso retornamos su pid, y si no imprimimos un mensaje de error y salimos del script:
        if filtered_lines != []:
            pid = filtered_lines[0].split()[1]
            return pid
        else:
            print("udp-sender process not found")
            sys.exit(3)
    # Si se ha producido una excepción, imprimimos el error y salimos del script:
    except Exception as error_description:
        print(f"Unexpected error: {error_description}")
        sys.exit(4)


def kill_udp_sender(pid):
    """ Finaliza el proceso asociado al pid que recibe como parámetro, e imprime el return code.
        Si se produce una excepción, imprime el error y sale del script.
    """
    try:
        # Finalizamos el proceso asociado al pid especificado como parámetro, e imprimimos el return code:
        result = subprocess.run(f"kill {pid}".split(), check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        print(f"ReturnCode: {result.returncode}")
    # Si se ha producido una excepción, imprimimos el error y salimos del script:
    except Exception as error_description:
        print(f"Unexpected error: {error_description}")
        sys.exit(5)



# --------------------------------------------------------------------------------------------
#                                          MAIN
# --------------------------------------------------------------------------------------------


def main():
    """
    """
    # Evaluamos si se ha enviado la cantidad correcta de parámetros, y en el formato correcto:
    check_params()

    # Obtenemos la ruta completa al archivo de imagen:
    file_path = build_file_path()

    # Si no existe el archivo de imagen, imprimimos un mensaje de error y salimos del script:
    if not os.path.exists(file_path):
        print("Image file doesn't exist")
        sys.exit(2)

    # Obtenemos el pid del proceso "udp-sender" asociado a la imagen especificada:
    pid = get_process_pid(file_path)

    # Finalizamos el proceso "udp-sender" encontrado:
    kill_udp_sender(pid)



# --------------------------------------------------------------------------------------------

if __name__ == "__main__":
    main()

# --------------------------------------------------------------------------------------------
